with ada.text_io;
use  ada.text_io;

with core, ui, effect, attribute, skill, resource, faction, might, magic, item, unit, construction, chad, world;

procedure main is

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

	side_panel     : integer := 480;
	preview_x      : integer := 24;
	preview_y      : integer := 24;
	preview_width  : integer := 0;
	preview_height : integer := 0;

	player : chad.information := chad.trait (chad.ognjen);

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

begin

	core.dash;
	core.echo (core.comment, "Copyright (C) 2024 -- Ognjen 'xolatile' Milan Robovic");
	core.echo (core.comment, "Version -- 1.0.0");
	core.echo (core.comment, "License -- GNU/GPLv3+");
	core.echo (core.comment, "Xhads is free software, you can redistribute it and modify it under the terms of the GNU General Public License by Free Software Foundation.");
	core.dash;

	core.configure;
	ui.configure;

	core.play_sound (core.import_sound (core.c_string ("./song/main_menu.ogg")));

	attribute.configure;
	skill.configure;
	resource.configure;
	might.configure;
	magic.configure;
	item.configure;
	unit.configure;
	construction.configure;
	chad.configure;
	world.configure;

	core.ai_synchronize (6);

	world.make (world.swamp, 80, 40);

	preview_width  := core.window_width - side_panel;
	preview_height := core.window_height;

	core.echo (core.success, "Successfully initialized game data, entering main gameplay loop.");

	gameplay: loop
		core.synchronize;
		--
		exit when core.engine_active = false;
		--
		if core.signal_mode = core.signal_code'pos (core.signal_a) then preview_width  := preview_width  - 60; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_d) then preview_width  := preview_width  + 60; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_w) then preview_height := preview_height - 60; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_s) then preview_height := preview_height + 60; end if;
		--
		if core.signal_mode = core.signal_code'pos (core.signal_left)  then core.camera.x := core.camera.x - 1; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_right) then core.camera.x := core.camera.x + 1; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_up)    then core.camera.y := core.camera.y - 1; end if;
		if core.signal_mode = core.signal_code'pos (core.signal_down)  then core.camera.y := core.camera.y + 1; end if;
		--
		if core.cursor_mode = 3 then ui.active := ui.default; else ui.active := ui.steam; end if;
		--
		core.camera.x := core.clip (core.camera.x, 0, world.map.width  - preview_width  / core.base);
		core.camera.y := core.clip (core.camera.y, 0, world.map.height - preview_height / core.base);
		--
		world.draw (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y, core.cursor_mode = 2);
		--
		--~core.draw_central_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--~core.draw_squared_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--~core.draw_hexagon_grid (preview_x, preview_y, preview_width - 2 * preview_x, preview_height - 2 * preview_y);
		--
		ui.draw_menu      (0,             0, preview_width, preview_height, false);
		ui.draw_tiny_menu (preview_width, 0, side_panel,    preview_height, true);
		--
		for this in magic.blow_away          .. magic.thunderclap            loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.chain_magic_arrow))      - 60, 112 * 0 + 64); end loop;
		for this in magic.agony_mass         .. magic.weakness               loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.thunderclap))            - 60, 112 * 1 + 64); end loop;
		for this in magic.earthquake         .. magic.summon_earth_elemental loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.weakness))               - 60, 112 * 2 + 64); end loop;
		for this in magic.determination_mass .. magic.summon_fire_elemental  loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.summon_earth_elemental)) - 60, 112 * 3 + 64); end loop;
		for this in magic.blindness          .. magic.sunburst               loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.summon_fire_elemental))  - 60, 112 * 4 + 64); end loop;
		for this in magic.dispel_magic_mass  .. magic.time_statis            loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.sunburst))               - 60, 112 * 5 + 64); end loop;
		for this in magic.blizzard           .. magic.winter_circle          loop magic.view (this, 112 * (magic.codex'pos (this) - magic.codex'pos (magic.time_statis))            - 60, 112 * 6 + 64); end loop;
		--
		--~for index in item.codex loop item.draw (index, 32 * (item.codex'pos (index) / 10) + 32, 32 * (item.codex'pos (index) mod 10) + 32); end loop;
		--~for index in attribute.codex loop attribute.draw (index, 32 * attribute.codex'pos (index) + 64,  64); end loop;
		--~for index in resource.codex  loop resource.draw  (index, 32 * resource.codex'pos  (index) + 64,  96); end loop;
		--~for index in skill.codex     loop skill.draw     (index, 32 * skill.codex'pos     (index) + 64, 128); end loop;
		--
		core.draw_state_box (preview_width + 32, 32);
		--
		attribute.menu (300, 200, false);
		skill.menu     (600, 300, true);
		resource.menu  (300, 500, false);
		unit.menu      (300, 500, true);
		--
		--~unit.stat (unit.griffin,    0, 0, true);
		--~unit.stat (unit.halberdier, 0, 0, true);
		--~unit.stat (unit.spirit,     0, 0, true);
		--~unit.stat (unit.power_lich, 0, 0, true);
		--
		--~might.menu (0, 0, true);
		--~magic.menu (0, 0, true);
		--
		ui.draw_text_box (0, core.window_height - 32, core.window_width, 32);
	end loop gameplay;

------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

end main;
